import json

from pysondb.cli import main


TEST_DATA = {
    'version': 2,
    'keys': [
        'age',
        'name',
        'place'
    ],
    'data': {
        '219520953066905460': {
            'name': 'ad0',
            'age': 0,
            'place': 'US'
        },
        '110180374400879352': {
            'name': 'ad1',
            'age': 1,
            'place': 'US'
        },
        '224980674034561069': {
            'name': 'ad2',
            'age': 7,
            'place': 'UK'
        }
    }
}


SUCCESS_TEST_OUTPUT = \
    """+--------------------+-----+------+-------+
|         id         | age | name | place |
+--------------------+-----+------+-------+
| 219520953066905460 |  0  | ad0  |   US  |
| 110180374400879352 |  1  | ad1  |   US  |
| 224980674034561069 |  7  | ad2  |   UK  |
+--------------------+-----+------+-------+
"""


def test_cli_show(tmpdir, capsys):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))

    assert main(('show', f.strpath)) == 0
    cap, _ = capsys.readouterr()
    assert str(cap) == str(SUCCESS_TEST_OUTPUT)


def test_cli_show_no_prettytable_installed(tmpdir, mocker, capsys):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    mocker.patch('pysondb.utils.PRETTYTABLE', False)

    assert main(('show', f.strpath)) == 1
    cap, _ = capsys.readouterr()
    assert cap == 'install prettytable (pip3 install prettytable) to run the following command\n'


def test_cli_show_v1_db(tmpdir, mocker, capsys):
    f = tmpdir.join('test.json')
    f.write(json.dumps({'data': [{'name': 'ad', 'age': 1, 'id': 353634357}]}))
    assert main(('show', f.strpath)) == 1
    cap, _ = capsys.readouterr()
    assert cap == 'the DB must be a v2 DB, you can use the migrate command to the convert your DB\n'



import json
from copy import deepcopy
from typing import Any
from typing import Dict

import pytest

from pysondb.db import PysonDB
from pysondb.errors import UnknownKeyError


TEST_DATA = {
    'version': 2,
    'keys': [
        'age',
        'name',
        'place'
    ],
    'data': {
        '219520953066905460': {
            'name': 'ad0',
            'age': 0,
            'place': 'US'
        },
        '110180374400879352': {
            'name': 'ad1',
            'age': 1,
            'place': 'US'
        },
        '224980674034561069': {
            'name': 'ad2',
            'age': 74574654,
            'place': 'UK'
        },
        '228563587602913112': {
            'name': 'ad3',
            'age': 3,
            'place': 'UK'
        },
        '167833310760833974': {
            'name': 'ad4',
            'age': 4,
            'place': 'IN'
        }
    }
}


def test_update_by_query(tmpdir):
    final_data: Dict[str, Any] = deepcopy(TEST_DATA)

    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)

    assert db.update_by_query(
        query=lambda x: x['place'] == 'US',
        new_data={'place': 'AU'}
    ) == ['219520953066905460', '110180374400879352']

    final_data['data']['219520953066905460']['place'] = 'AU'
    final_data['data']['110180374400879352']['place'] = 'AU'

    assert json.loads(f.read()) == final_data


def test_update_by_query_unknown_key_error(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)

    with pytest.raises(UnknownKeyError):
        db.update_by_query(
            query=lambda x: x['place'] == 'IN',
            new_data={'summer': True}
        )


@pytest.mark.parametrize(
    'query',
    (
        'a',
        [1, 2],
        (1, 3),
        {'1, 3'},
        {'a': 4, 'b': 5}
    )
)
def test_update_by_query_type_error(tmpdir, query):
    f = tmpdir.join('test.json')
    db = PysonDB(f.strpath)

    with pytest.raises(TypeError):
        db.update_by_query(query, {'name': 'ad'})


@pytest.mark.parametrize(
    'data',
    (
        [1, 2],
        1,
        'string',
        (1, 3),
        {1, 3}
    )
)
def test_new_data_type_error(tmpdir, data):
    f = tmpdir.join('test.json')
    db = PysonDB(f.strpath)

    with pytest.raises(TypeError):
        db.update_by_query(lambda _: True, data)



import json

import pytest

from pysondb.db import PysonDB
from pysondb.errors import IdDoesNotExistError


TEST_DATA = {
    'version': 2,
    'keys': ['age', 'name'],
    'data': {
        '2352346': {
            'age': 4,
            'name': 'mathew_first'
        },
        '1234567': {
            'age': 9,
            'name': 'new_user'
        }
    }
}


def test_get_by_id(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA, indent=4))
    db = PysonDB(f.strpath)

    assert db.get_by_id('2352346') == {'age': 4, 'name': 'mathew_first'}


def test_get_by_id_id_does_no_exist_error(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA, indent=4))
    db = PysonDB(f.strpath)

    with pytest.raises(IdDoesNotExistError):
        db.get_by_id('1212121')


@pytest.mark.parametrize(
    'data',
    (
        2235235,
        [1, 2],
        (1, 2, 3),
        {1, 2, 3},
        {'2': '4'}
    )
)
def test_get_id_type_error(tmpdir, data):
    f = tmpdir.join('test.json')
    db = PysonDB(f.strpath)

    with pytest.raises(TypeError):
        db.get_by_id(data)



import json

from pysondb.cli import main


TEST_DATA = {
    'version': 2,
    'keys': [
        'age',
        'name',
        'place'
    ],
    'data': {
        '219520953066905460': {
            'name': 'ad0',
            'age': 0,
            'place': 'US'
        },
        '110180374400879352': {
            'name': 'ad1',
            'age': 1,
            'place': 'US'
        },
        '224980674034561069': {
            'name': 'ad2',
            'age': 7,
            'place': 'UK'
        }
    }
}


def test_cli_purge(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))

    assert json.loads(f.read()) == TEST_DATA
    assert main(('purge', f.strpath)) == 0
    assert json.loads(f.read()) == {'version': 2, 'data': {}, 'keys': []}



import json

from pysondb.db import PysonDB


FINAL_DATA = {
    'version': 2,
    'keys': ['age', 'name'],
    'data': {
        '1': {
            'age': 4,
            'name': 'mathew_first'
        },
        '2': {
            'age': 9,
            'name': 'new_user'
        }
    }
}


def get_nums(n):
    yield str(n)
    yield from get_nums(n + 1)


def test_id_generator_incremental_id(tmpdir):
    f = tmpdir.join('test.json')
    nums = get_nums(1)
    db = PysonDB(f.strpath)
    db.set_id_generator(lambda: next(nums))
    assert db.add_many([
        {'age': 4, 'name': 'mathew_first'},
        {'age': 9, 'name': 'new_user'}
    ], json_response=True) == {'1': {'age': 4, 'name': 'mathew_first'}, '2': {'age': 9, 'name': 'new_user'}}
    assert json.loads(f.read()) == FINAL_DATA



import json
from copy import deepcopy
from typing import Any
from typing import Dict

import pytest

from pysondb.db import PysonDB
from pysondb.errors import IdDoesNotExistError


TEST_DATA = {
    'version': 2,
    'keys': [
        'age',
        'name',
        'place'
    ],
    'data': {
        '219520953066905460': {
            'name': 'ad0',
            'age': 0,
            'place': 'US'
        },
        '110180374400879352': {
            'name': 'ad1',
            'age': 1,
            'place': 'US'
        },
        '224980674034561069': {
            'name': 'ad2',
            'age': 74574654,
            'place': 'UK'
        },
        '228563587602913112': {
            'name': 'ad3',
            'age': 3,
            'place': 'UK'
        },
        '167833310760833974': {
            'name': 'ad4',
            'age': 4,
            'place': 'IN'
        }
    }
}


def test_delete_by_id(tmpdir):
    final_data: Dict[str, Any] = deepcopy(TEST_DATA)
    del final_data['data']['110180374400879352']

    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)

    db.delete_by_id('110180374400879352')
    assert json.loads(f.read()) == final_data


def test_delete_by_id_id_not_found_error(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)

    with pytest.raises(IdDoesNotExistError):
        db.delete_by_id('2345')



import json

import pytest

from pysondb.db import PysonDB


TEST_DATA = {
    'version': 2,
    'keys': [
        'age',
        'name',
        'place'
    ],
    'data': {
        '219520953066905460': {
            'name': 'ad0',
            'age': 0,
            'place': 'US'
        },
        '110180374400879352': {
            'name': 'ad1',
            'age': 1,
            'place': 'US'
        },
        '224980674034561069': {
            'name': 'ad2',
            'age': 7,
            'place': 'UK'
        },
        '228563587602913112': {
            'name': 'ad3',
            'age': 3,
            'place': 'UK'
        },
        '167833310760833974': {
            'name': 'ad4',
            'age': 4,
            'place': 'IN'
        }
    }
}


def test_delete_by_query(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)

    assert db.delete_by_query(
        query=lambda x: x['age'] < 6
    ) == ['219520953066905460', '110180374400879352', '228563587602913112', '167833310760833974']
    assert json.loads(f.read()) == {'version': 2, 'keys': ['age', 'name', 'place'], 'data': {
        '224980674034561069': {'name': 'ad2', 'age': 7, 'place': 'UK'}}}


@pytest.mark.parametrize(
    'query',
    (
        {'a': '4'},
        [1, 2],
        1,
        '2',
        {1, 2}
    )
)
def test_delete_by_query_type_error(tmpdir, query):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)

    with pytest.raises(TypeError):
        db.delete_by_query(query)



import json
from pathlib import Path

from pysondb.db import PysonDB
from pysondb.db_types import DBSchemaType


SAMPLE_DATA: DBSchemaType = {
    'version': 2,
    'keys': ['a', 'b', 'c'],
    'data': {
        '384753047545745': {
            'a': 1,
            'b': 'something',
            'c': True
        }
    }
}


def test_load(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(SAMPLE_DATA))

    db = PysonDB(f.strpath)
    assert db._load_file() == SAMPLE_DATA


def test_dump(tmpdir):
    f = tmpdir.join('test.py')

    db = PysonDB(f.strpath)
    db._dump_file(SAMPLE_DATA)
    assert f.read() == json.dumps(SAMPLE_DATA, indent=4)


def test_gen_file_file_exists(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(SAMPLE_DATA, indent=4))
    PysonDB(f)

    assert f.read() == json.dumps(SAMPLE_DATA, indent=4)


def test_gen_file_file_does_not_exist(tmp_path):
    filename = tmp_path / 'test.json'
    PysonDB(filename)

    assert Path(filename).is_file()
    assert filename.read_text() == json.dumps(
        {'version': 2, 'keys': [], 'data': {}}, indent=4)



import json
from pathlib import Path

from pysondb.db import PysonDB


TEST_FINAL_DATA = {
    'version': 2,
    'keys': ['age', 'name'],
    'data': {
        '0': {
            'age': 3,
            'name': 'test'
        }
    }
}


def _new_gen_id(n):
    yield str(n)
    yield from _new_gen_id(n + 1)


def test_autoupdate(tmpdir, mocker):
    f = tmpdir.join('test.json')
    _ids = _new_gen_id(0)
    mocker.patch('pysondb.db.PysonDB._gen_id', wraps=lambda: next(_ids))
    db = PysonDB(f.strpath, auto_update=False)
    db.add({
        'name': 'test', 'age': 3
    })

    assert db.auto_update is False
    assert db._au_memory == TEST_FINAL_DATA
    assert Path(f.strpath).is_file() is False


def test_autoupdate_force_load(tmpdir, mocker):
    f = tmpdir.join('test.json')
    _ids = _new_gen_id(0)
    mocker.patch('pysondb.db.PysonDB._gen_id', wraps=lambda: next(_ids))
    f.write(json.dumps(TEST_FINAL_DATA))
    db = PysonDB(f.strpath, auto_update=False)

    db.force_load()
    assert db.auto_update is False
    assert db._au_memory == TEST_FINAL_DATA


def test_autoupdate_commit(tmpdir, mocker):
    f = tmpdir.join('test.json')
    _ids = _new_gen_id(0)
    mocker.patch('pysondb.db.PysonDB._gen_id', wraps=lambda: next(_ids))
    db = PysonDB(f.strpath, auto_update=False)
    db.add({
        'name': 'test', 'age': 3
    })
    db.commit()

    assert db.auto_update is False
    assert json.loads(f.read()) == TEST_FINAL_DATA


def test_autoupdate_accidental_commit(tmpdir):
    f = tmpdir.join('test.json')
    db = PysonDB(f.strpath)
    db.commit()

    assert db.auto_update is True


def test_autoupdate_accidental_force_load(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_FINAL_DATA))

    db = PysonDB(f.strpath)
    db.force_load()

    assert db.auto_update is True



import json

import pytest

from pysondb.db import PysonDB
from pysondb.errors import UnknownKeyError


TEST_DATA = [
    {
        'name': 'ad',
        'age': 19
    },
    {
        'name': 'fredy',
        'age':  69
    },
    {
        'name': 'mathew',
        'age': 69
    }
]

INITIAL_TEST_DATA = {
    'version': 2,
    'keys': ['age', 'name'],
    'data': {
        '2352346': {
            'age': 4,
            'name': 'mathew_first'
        }
    }
}


def _new_gen_id(n):
    yield str(n)
    yield from _new_gen_id(n + 1)


def test_add_many_empty_file(tmpdir, mocker):

    final_data = {
        'version': 2,
        'keys': ['age', 'name'],
        'data': {
            '0': {
                'name': 'ad',
                'age': 19
            },
            '1': {
                'name': 'fredy',
                'age':  69
            },
            '2': {
                'name': 'mathew',
                'age': 69
            }
        }
    }

    _ids = _new_gen_id(0)
    mocker.patch('pysondb.db.PysonDB._gen_id', wraps=lambda: next(_ids))

    f = tmpdir.join('test.json')
    db = PysonDB(f.strpath)
    db.add_many(TEST_DATA)
    assert json.loads(f.read()) == final_data


def test_add_many_non_empty_file(tmpdir, mocker):
    final_data = {
        'version': 2,
        'keys': ['age', 'name'],
        'data': {
            '2352346': {
                'age': 4,
                'name': 'mathew_first'
            },
            '0': {
                'name': 'ad',
                'age': 19
            },
            '1': {
                'name': 'fredy',
                'age':  69
            },
            '2': {
                'name': 'mathew',
                'age': 69
            }
        }
    }

    _ids = _new_gen_id(0)
    mocker.patch('pysondb.db.PysonDB._gen_id', wraps=lambda: next(_ids))
    f = tmpdir.join('test.json')
    f.write(json.dumps(INITIAL_TEST_DATA, indent=4))

    db = PysonDB(f.strpath)
    db.add_many(TEST_DATA)
    assert json.loads(f.read()) == final_data


@pytest.mark.parametrize(
    'data',
    (
        (1,),
        {'a': '3'},
        {1, 2}
    )
)
def test_add_many_type_error_for_list(tmpdir, data):
    f = tmpdir.join('test.json')
    db = PysonDB(f.strpath)

    with pytest.raises(TypeError):
        db.add_many(data)


@pytest.mark.parametrize(
    'data',
    (
        [1, 2],
        ['a', 'b'],
        [[1, 2], [3, 4]],
        [(1, 2), (3)],
        [{1, 2}]
    )
)
def test_add_many_type_error_for_dict_in_list(tmpdir, data):
    f = tmpdir.join('test.json')
    db = PysonDB(f.strpath)

    with pytest.raises(TypeError):
        db.add_many(data)


def test_add_many_unknown_key_error(tmpdir):
    f = tmpdir.join('test.json')
    db = PysonDB(f.strpath)

    with pytest.raises(UnknownKeyError):
        db.add_many(
            [
                {'name': 'ad', 'age': 4},
                {'name': 'test', 'age': 2},
                {'name': 'new_ad', 'age': 5, 'place': 'GB'}
            ]
        )


@pytest.mark.parametrize(
    'data',
    (
        {'name': 'ad', 'age': 4, 'place': 'GB'},
        {'name': 'ad'},
        {'place': 'GB', 'is_alive': True}
    )
)
def test_add_many_unknown_key_error_non_empty_file(tmpdir, data):
    f = tmpdir.join('test.json')
    f.write(json.dumps(INITIAL_TEST_DATA, indent=4))
    db = PysonDB(f.strpath)

    with pytest.raises(UnknownKeyError):
        db.add_many([
            data
        ])


@pytest.mark.parametrize(
    'data',
    (
        '',
        [],
        {}
    )
)
def test_add_many_empty_data(tmpdir, data):
    f = tmpdir.join('test.json')
    db = PysonDB(f.strpath)
    db.add_many(data)


def test_add_many_json_response(tmpdir, mocker):
    response = {
        '0': {
            'name': 'ad',
            'age': 19
        },
        '1': {
            'name': 'fredy',
            'age':  69
        },
        '2': {
            'name': 'mathew',
            'age': 69
        }
    }

    f = tmpdir.join('test.json')
    _ids = _new_gen_id(0)
    mocker.patch('pysondb.db.PysonDB._gen_id', wraps=lambda: next(_ids))

    db = PysonDB(f.strpath)
    assert db.add_many(TEST_DATA, json_response=True) == response



import json

from pysondb.cli import main

TEST_DATA_1 = {
    'version': 2,
    'keys': ['test'],
    'data': {
        '211194894507061604': {
            'test': '3'
        },
        '107314111299174914': {
            'test': '4'
        }
    }
}


TEST_DATA_2 = {
    'version': 2,
    'keys': ['test'],
    'data': {
        '211194894507061605': {
            'test': '3'
        },
        '107314111299174916': {
            'test': '4'
        }
    }
}


TEST_DATA_ERR_KEYS = {
    'version': 2,
    'keys': ['age'],
    'data': {
        '211194894507061605': {
            'age': '3'
        },
        '107314111299174916': {
            'age': '4'
        }
    }
}

FINAL_DATA = {
    'version': 2,
    'keys': ['test'],
    'data': {
        '211194894507061604': {
            'test': '3'
        },
        '107314111299174914': {
            'test': '4'
        },
        '211194894507061605': {
            'test': '3'
        },
        '107314111299174916': {
            'test': '4'
        }
    }
}


def test_cli_merge(tmpdir, capsys):
    f1 = tmpdir.join('test1.json')
    f2 = tmpdir.join('test2.json')
    f3 = tmpdir.join('new_test.json')

    f1.write(json.dumps(TEST_DATA_1))
    f2.write(json.dumps(TEST_DATA_2))

    assert main(('merge', f1.strpath, f2.strpath, '-o', f3.strpath)) == 0
    cap, _ = capsys.readouterr()
    assert cap == "DB\'s merged successfully\n"
    assert json.loads(f3.read()) == FINAL_DATA


def test_cli_merge_key_mismatch(tmpdir, capsys):
    f1 = tmpdir.join('test1.json')
    f2 = tmpdir.join('test2.json')

    f1.write(json.dumps(TEST_DATA_1))
    f2.write(json.dumps(TEST_DATA_ERR_KEYS))

    assert main(('merge', f1.strpath, f2.strpath, '-o', 'test')) == 1
    cap, err = capsys.readouterr()
    assert err == "All the DB\'s must have the same keys\n"
    assert cap == ''



from typing import Any
from typing import Callable
from typing import Dict
from typing import List
from typing import Union


DBSchemaType = Dict[
    str, Union[
        int,
        List[str],
        Dict[
            str, Any
        ]
    ]
]

SimpleTypeGroup = Union[int, str, bool]

SingleDataType = Dict[
    str, Union[
        int,
        str,
        bool,
        List[SimpleTypeGroup]
    ]
]

QueryType = Callable[[Dict[str, Any]], bool]
IdGeneratorType = Callable[[], str]
ReturnWithIdType = Dict[
    str, Dict[
        str, SimpleTypeGroup
    ]
]
NewKeyValidTypes = Union[List, Dict, str, int, bool]



import json
from copy import deepcopy

import pytest

from pysondb.db import PysonDB
from pysondb.errors import SchemaTypeError
from pysondb.errors import UnknownKeyError


TEST_DATA = {
    'version': 2,
    'keys': ['age', 'name'],
    'data': {
        '2352346': {
            'age': 4,
            'name': 'mathew'
        }
    }
}


def test_add_empty_file(tmpdir, mocker):
    mocker.patch('pysondb.db.PysonDB._gen_id', return_value='2352346')
    f = tmpdir.join('test.json')
    db = PysonDB(f.strpath)

    assert db.add({'age': 4, 'name': 'mathew'}) == '2352346'
    assert json.loads(f.read()) == TEST_DATA


def test_add_non_empty_file(tmpdir, mocker):
    final_data = {
        'version': 2,
        'keys': ['age', 'name'],
        'data': {
            '2352346': {
                'age': 4,
                'name': 'mathew'
            },
            '1234567': {
                'age': 18,
                'name': 'ad'
            }
        }
    }

    mocker.patch('pysondb.db.PysonDB._gen_id', return_value='1234567')
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA, indent=4))

    db = PysonDB(f.strpath)
    assert db.add({'name': 'ad', 'age': 18}) == '1234567'
    assert json.loads(f.read()) == final_data


def test_add_unknown_key_error(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))

    db = PysonDB(f.strpath)
    with pytest.raises(UnknownKeyError):
        db.add({'age': 4, 'name': 'fredy', 'place': 'GB'})


def test_schema_type_error(tmpdir):
    new_test_data = deepcopy(TEST_DATA)
    new_test_data['keys'] = 'test'

    f = tmpdir.join('test.json')
    f.write(json.dumps(new_test_data, indent=4))
    db = PysonDB(f.strpath)
    with pytest.raises(SchemaTypeError):
        db.add({'name': 'test', 'age': 69})


def test_add_keys_mismatched_length(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))

    db = PysonDB(f.strpath)
    with pytest.raises(UnknownKeyError):
        db.add({'name': 'test'})


@pytest.mark.parametrize(
    'data',
    (
        [0, 2],
        1,
        'hello',
        (1, 2),
        type('Foo', (), {})
    )
)
def test_add_type_error(tmpdir, data):
    f = tmpdir.join('test.json')
    db = PysonDB(f.strpath)
    with pytest.raises(TypeError):
        db.add(data)



from pysondb.cli import main


def test_cli_info_ujson_exist(mocker, capsys):
    mocker.patch('pysondb.db.UJSON', True)

    assert main(('--info',)) == 0
    out, _ = capsys.readouterr()
    assert out == "PysonDB - 2.0.0\nusing 'ujson' JSON parser\n"


def test_cli_info_ujson_does_not_exist(mocker, capsys):
    mocker.patch('pysondb.db.UJSON', False)

    assert main(('--info',)) == 0
    out, _ = capsys.readouterr()
    assert out == 'PysonDB - 2.0.0\nusing builtin JSON parser\n'



import json
import pytest
from pysondb.db import PysonDB
from pysondb.errors import UnknownKeyError


TEST_DATA = {
    'version': 2,
    'keys': ['age', 'name', 'toy'],
    'data': {
        '2352346': {
            'age': 4,
            'name': 'mathew_first',
            'toy': 'car',
        },
        '1234567': {
            'age': 9,
            'name': 'new_user',
            'toy': 'ball',
        }
    }
}


def test_get_all_select_keys(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))

    db = PysonDB(f.strpath)
    assert db.get_all_select_keys(['age']) == {
        '2352346': {
            'age': 4,
        },
        '1234567': {
            'age': 9,
        }
    }
    assert db.get_all_select_keys(['name', 'toy']) == {
        '2352346': {
            'name': 'mathew_first',
            'toy': 'car',
        },
        '1234567': {
            'name': 'new_user',
            'toy': 'ball',
        }
    }


def test_get_all_select_keys_empty_file(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps({'version': 2, 'keys': [], 'data': {}}))

    db = PysonDB(f.strpath)
    assert db.get_all_select_keys([]) == {}

def test_get_all_select_keys_wrong_key(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))

    db = PysonDB(f.strpath)
    with pytest.raises(UnknownKeyError):
        db.get_all_select_keys(['wrong_key'])



import json

import pytest

from pysondb.db import PysonDB


TEST_DATA = {
    'version': 2,
    'keys': ['age', 'name'],
    'data': {
        '2352346': {
            'age': 4,
            'name': 'mathew_first'
        },
        '1234567': {
            'age': 9,
            'name': 'new_user'
        }
    }
}


def test_add_new_key(tmpdir):

    DATA_SUCCESS = {
        'version': 2,
        'keys': ['age', 'name', 'place'],
        'data': {
            '2352346': {
                'age': 4,
                'name': 'mathew_first',
                'place': 'GB'
            },
            '1234567': {
                'age': 9,
                'name': 'new_user',
                'place': 'GB'
            }
        }
    }

    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)
    db.add_new_key('place', 'GB')

    assert json.loads(f.read()) == DATA_SUCCESS


def test_add_new_key_no_default(tmpdir):
    DATA_SUCCESS = {
        'version': 2,
        'keys': ['age', 'name', 'place'],
        'data': {
            '2352346': {
                'age': 4,
                'name': 'mathew_first',
                'place': None
            },
            '1234567': {
                'age': 9,
                'name': 'new_user',
                'place': None
            }
        }
    }

    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)
    db.add_new_key('place')

    assert json.loads(f.read()) == DATA_SUCCESS


@pytest.mark.parametrize(
    'default',
    (
        (1,),
        type('test', (), {}),
    )
)
def test_add_new_key_invalid_data_type(tmpdir, default):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)

    with pytest.raises(TypeError):
        db.add_new_key(default)



import json

from pysondb.utils import migrate


def test_migrate():
    with open('tests/test_utils_migrate/old_db.json') as f:
        old = json.load(f)

    with open('tests/test_utils_migrate/new_data.json') as f:
        new = json.load(f)

    assert migrate(old) == new


def test_migrate_empty_data():
    assert migrate({'data': []}) == {'version': 2, 'keys': [], 'data': {}}



import json

import pytest

from pysondb.db import PysonDB
from pysondb.errors import IdDoesNotExistError
from pysondb.errors import UnknownKeyError


TEST_DATA = {
    'version': 2,
    'keys': ['age', 'name'],
    'data': {
        '2352346': {
            'age': 4,
            'name': 'mathew_first'
        },
        '1234567': {
            'age': 9,
            'name': 'new_user'
        }
    }
}


def test_update_by_id(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)

    db.update_by_id('1234567', {'age': 69})
    assert json.loads(f.read()) == {'version': 2, 'keys': ['age', 'name'], 'data': {'2352346': {
        'age': 4, 'name': 'mathew_first'}, '1234567': {'age': 69, 'name': 'new_user'}}}


def test_update_by_id_id_does_not_exists(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)

    with pytest.raises(IdDoesNotExistError):
        db.update_by_id('23526556', {'age': 69})


def test_update_by_id_unknown_key_error(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)

    with pytest.raises(UnknownKeyError):
        db.update_by_id('534535', {'place': 'GB'})



import json

import pytest

from pysondb.db import PysonDB


TEST_DATA = {
    'version': 2,
    'keys': ['age', 'name'],
    'data': {
        '2352346': {
            'age': 4,
            'name': 'mathew_first'
        },
        '1234567': {
            'age': 9,
            'name': 'new_user'
        }
    }
}


@pytest.mark.parametrize(
    'condition,output',
    (
        (lambda x: x['age'] >= 4, {'2352346': {'age': 4, 'name': 'mathew_first'},
         '1234567': {'age': 9, 'name': 'new_user'}}),
        (lambda x: x['name'] == 'new_user', {
         '1234567': {'age': 9, 'name': 'new_user'}}),
        (lambda x: x['age'] >= 4 and x['name'] == 'mathew_first',
         {'2352346': {'age': 4, 'name': 'mathew_first'}})
    )
)
def test_get_by_query(tmpdir, condition, output):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))
    db = PysonDB(f.strpath)

    assert db.get_by_query(condition) == output


def test_get_by_query_no_matches(tmpdir):
    pass


@pytest.mark.parametrize(
    'condition',
    (
        [1, 2],
        {'age': 4, 'name': 3}
    )
)
def test_get_by_query_type_error(tmpdir, condition):
    f = tmpdir.join('test.json')
    db = PysonDB(f.strpath)

    with pytest.raises(TypeError):
        db.get_by_query(condition)



import json

from pysondb.db import PysonDB


TEST_DATA = {
    'version': 2,
    'keys': ['age', 'name'],
    'data': {
        '2352346': {
            'age': 4,
            'name': 'mathew_first'
        },
        '1234567': {
            'age': 9,
            'name': 'new_user'
        }
    }
}


def test_get_all(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps(TEST_DATA))

    db = PysonDB(f.strpath)
    assert db.get_all() == {
        '2352346': {
            'age': 4,
            'name': 'mathew_first'
        },
        '1234567': {
            'age': 9,
            'name': 'new_user'
        }
    }


def test_get_all_empty_file(tmpdir):
    f = tmpdir.join('test.json')
    f.write(json.dumps({'version': 2, 'keys': [], 'data': {}}))

    db = PysonDB(f.strpath)
    assert db.get_all() == {}
